# SUPPLEMENT #2 - R simulation code for predicting annual bycatch and mortality using WinBUGS model estimates.
# Summer L. Martin (1,2), Stephen M. Stohs (2) & Jeffrey E. Moore (2)
# (1) Scripps Insititution of Oceanography and (2) NOAA Southwest Fisheries Science Center
# email: s2martin@ucsd.edu, summerLmartin@gmail.com
# Date: July 30, 2014

# code to predict unobserved bycatch using WinBUGS model output (posterior parameter distributions) 
# Created: January 10, 2014

################### -------- PREDICTION ---------- #####################
# - select model to be used for prediction (e.g. M1z) and species (DC or MN)
# - access/save WinBUGS parameter estimates for that model
# - create a data frame to store prediction results
# - for one season, do the following steps 10,000 times:
# - randomly draw a value of each of the estimated parameters from the posterior vectors (1 per chain) produced by MCMC
# - plug parameters into equation for theta (if parameter isn't theta itself)
# - plug theta into equation for lambda
# - plug lambda into poisson equation for number of takes 'y'
# - randomly draw a mortality rate 'q' from beta-binomial distribution
# - multiply mortality rate 'q' by lambda to get the simulated draw from the posterior predictive distribution of the rate of mortality/serious injury
# - Plug 'q' and 'y' into Binomial distribution to get number of mortalities 'z' given number of takes 'y'
# - Create Vectors for Hypothetical future season *expected annual mortality* prediction (using avg total sets 2005-2009) -- CALCULATION FOR ONE YEAR ONLY
# - CALCULATE CUMULATIVE PROBABILITY CURVE TO GET 99% UPPER LIMIT on TAKES, MORTALITY RATE, and MORTALITIES

# ctrl +'L' to clear R console
# rm(list=ls())                      # clear files from R session to start with clean slate
# setwd("C:/Users/slm2.NMFS/Documents/Summer's documents/1 NOAA/3 Leatherback Bycatch Project/2 PAPER/Analysis")
# source("Data import and manipulation.r")
# library(R2WinBUGS)
# library(coda)
# library(stats)

# load("C:/Users/slm2.NMFS/Documents/Summer's documents/1 NOAA/3 Leatherback Bycatch Project/2 PAPER/Analysis/FinalModels/ModelRuns_06Jan2014.RData")
# setwd("C:/Users/slm2.NMFS/Documents/Summer's documents/1 NOAA/3 Leatherback Bycatch Project/2 PAPER/Analysis/FinalModels")
# source("cumul prob function.r")     # define function to calculate cumulative probabilities (***SEE BOTTOM OF SCRIPT FILE FOR THIS FUNCTION DEFINITION)

# FUNCTION DEFINITION: CALCULATE CUMULATIVE PROBABILITY CURVE TO GET 99% UPPER LIMIT on the number of takes, mortalities, mortality rate, etc....
# takes a vector of values, finds the unique values, sorts them, calculates the frequency of each unique value in the original vector of values,
# calculates the cumulative probability at each value in the sorted vector of unique values
# returns a list of 3 elements: 1) the sorted vector of unique values, 2) the cumulative probability vector corresponding to the sorted vector of unique values, 3) the upper 99% credibility interval limit of the value
cumul.prob.fxn <- function (vec.vals, s) {                            # arguments: vec.vals= vector of values for which to create cumul prob vector; s= number of simulation runs
  vec.vals.uniq <- sort(unique(vec.vals))                        # sort all unique values in vec.vals and sort from lowest to highest
  L <- length(vec.vals.uniq)                                     # length of vec.vals.uniq
  freq.i <- vector(mode="numeric", length=L)                     # vector to store frequency of each of the vec.vals.uniq values in the larger vec.vals
  prob.i <- vector(mode="numeric", length=L)                     # vector to store probability of y.sim.obs values across all simulation runs
  for (i in 1:L) {                                               # loop through length of vec.vals.uniq
    indxs.i <- which(vec.vals == vec.vals.uniq[i])              # get the index of each element in vec.vals that equals the 'ith' value in the unique sequence vec.vals.uniq
    freq.i[i] <- length(indxs.i)                                # count number of times the ith unique value occurs in the raw vec.vals vector
    prob.i[i] <- freq.i[i]/s                                    # probability of the unique value is its frequency in the vec.vals vector divided by the number of simulation runs, s
  }
  cumul.prob <- vector(mode="numeric", length=L)                 # initialize vector that will store cumulative probability for each value in vec.vals.uniq
  cumul.prob[1] <- prob.i[1]                                     # first element is just the probability of the first value in vec.vals.uniq
  for (j in 2:L) {                                               # loop through sorted unique values in vec.vals.uniq and calculate cumulative probability of vec.vals being that value or less
    cumul.prob[j] <- sum(prob.i[1:j])                           # fill in cumulative prob vector by summing all probs for y's up to jth y
  }
  up99CIlimit <- vec.vals.uniq[min(which(cumul.prob>=0.99))]            # 99% CI upper bound
  median50 <- vec.vals.uniq[min(which(cumul.prob>=0.50))]          # 50% point = median/representative/central tendency
  output <- list(vec.vals.uniq, prob.i,cumul.prob, up99CIlimit, median50)         # create a list of 4 elements to return
  names(output) <- c("Values", "Probs","CumulProbs", "UpperLim99CI", "Median50")    # name the elements in the list
  return(output)                                                        # return the list as the output of the function
}
####  

# RUN THIS tracer command immediately after running the model of choice for species of choice using Supplement 1 code (estimating model parameters) 
# these lines below need the saved MCMC chain outputs from the WinBUGS estimation output
tracer <- mcmc.tracer                    # shortens name for ease of use here
len.tra <- length(tracer[[1]][,1])       # used to be 10,000 -- b/c 10,000 stored values in the MCMC posterior vectors
s <- 10000     # s = number of simulation runs (number of times to randomly draw betas, put them into thetas, put those into lambda, and pull a nmbr of takes 'y' from its Poisson distribution characterized by lambda)

mdl <- "aM2"   # specify model ("aM1r" for leatherbacks; "aM2" for humpbacks)
sp <- "DC"     # DC leatherbacks or MN humpbacks

# *** CREATE LISTS FOR STORAGE (each with n.seasons elements, s simulation runs within each element)
# "s" here refers to "simulated"
b0.s <- vector(mode="list", length=n.seasons)         
b1.s <- vector(mode="list", length=n.seasons)
b2.s <- vector(mode="list", length=n.seasons)
b3.s <- vector(mode="list", length=n.seasons) 

p1.s <- vector(mode="list", length=n.seasons)         
p2.s <- vector(mode="list", length=n.seasons)
p3.s <- vector(mode="list", length=n.seasons)
p4.s <- vector(mode="list", length=n.seasons) 

theta1.s <- vector(mode="list", length=n.seasons)
theta2.s <- vector(mode="list", length=n.seasons)
theta3.s <- vector(mode="list", length=n.seasons)
theta4.s <- vector(mode="list", length=n.seasons)
theta.s <- vector(mode="list", length=n.seasons)

lam.s.obs <- vector(mode="list", length=n.seasons)
lam.s.unobs <- vector(mode="list", length=n.seasons)
lam.s.tot <- vector(mode="list", length=n.seasons)
y.s.obs <- vector(mode="list", length=n.seasons)
y.s.unobs <- vector(mode="list", length=n.seasons)
q.sim <- vector(mode="list", length=n.seasons)
mr.s.obs <- vector(mode="list", length=n.seasons)
mr.s.unobs <- vector(mode="list", length=n.seasons)
mr.s.tot <- vector(mode="list", length=n.seasons)
z.s.obs <- vector(mode="list", length=n.seasons)
z.s.unobs <- vector(mode="list", length=n.seasons)
lam.s.HypoFutSsn <- vector(mode="numeric", length=s)    # to store values of 'z' each simulation pass (z = nmbr of unobserved mortalities, binomially distributed based on number of draws, y takes, and q probability of mortality)
mr.s.HypoFutSsn <- vector(mode="numeric", length=s)     # to store values of 'z' each simulation pass (z = nmbr of unobserved mortalities, binomially distributed based on number of draws, y takes, and q probability of mortality)

# Create a data frame that will store the results from each year (summaries from y takes and z mortalities)
predictions.df <- as.data.frame(matrix(nrow=n.seasons, ncol=31))                           # initialize data frame to store results
# name columns of predictions.df
colnames(predictions.df) <- c("Season", "ObsvdSets", "UnobsvdSets", "ObsvdTakes", "ObsvdMorts", 
                              "Theta99", "Theta50","LamObs99", "LamObs50","LamUnobs99", "LamUnobs50", "LamTot99", "LamTot50",
                              "TakesObs99", "TakesObs50", "TakesUnobs99","TakesUnobs50", "TakesTotal99", "TakesTotal50", 
                              "MortRateObs99", "MortRateObs50", "MortRateUnobs99", "MortRateUnobs50", "MortRateTotal99","MortRateTotal50", 
                              "MortsObs99", "MortsObs50", "MortsUnobs99", "MortsUnobs50", "MortsTotal99", "MortsTotal50") 

obs.sets   <- effort.summ.df[ ,2:5]
unobs.sets <- effort.summ.df[ ,6:9]
tot.sets   <- effort.summ.df[ ,10:13]

# ********** model specs **********
# breaks up the data into categories that are inside/outside of the model variable
# for example, model 2 ('aM2') contains the AREA variable, and 
# columns 1 & 3 in the data fall OUTSIDE the closure area 
# while data in columns 2 & 4 fall INSIDE the closure area 

if (mdl == "aM2" | mdl == "aM2z") {
  OUT.ctgs <- c(1,3)
  IN.ctgs <- c(2,4)
}

if (mdl == "aM3" | mdl == "aM3z") {
  OUT.ctgs <- c(1,2)
  IN.ctgs <- c(3,4)
}

if (mdl == "aM4" | mdl == "aM4z") {
  OUT.ctgs <- c(1,2,3)
  IN.ctgs <- 4
}

# ****************************
# ****************************
# PREDICTIONS by season (1990-2009)
for (yr in 1:n.seasons) {                               # yr = 1 is 1990, yr = 2 is 1991, etc.                      
  p.season  <- effort.summ.df$Season[yr]                # p.season = prediction season
#** Model M1, M1z and M1r
  if (mdl=="aM1"|mdl=="aM1z"|mdl=="aM1r") {
    obs.sets.ALL <- sum(obs.sets[yr, ])       # observed sets for the year
    unobs.sets.ALL <- sum(unobs.sets[yr, ])   # unobserved sets for the year
    tot.sets.ALL <- obs.sets.ALL + unobs.sets.ALL  # observed + unobserved sets for the year
    # calculate recent 5-year average effort in each category  
    Ntot.avg <- round(sum(tot.sets[16:20,])/5)  # average *total* effort (nmbr of sets) from 2005 through 2009 (conservative estimate of avg total effort in future seasons)  
  }    
#** Models M2, M2z, M3, M3z, M4, M4z - data split into 2 categories (based on combos of C1, C2, C3, C4) ###  
  if (mdl=="aM2"|mdl=="aM2z"|mdl=="aM3"|mdl=="aM3z"|mdl=="aM4"|mdl=="aM4z") {
    # observed sets broken out by category for the year
    obs.sets.OUT <- sum(obs.sets[yr,OUT.ctgs])   # outside specifications (time/area): nmbr of unobs sets that occurred in p.season (over which to predict)
    obs.sets.IN <-  sum(obs.sets[yr,IN.ctgs])    # inside specifications (time/area): nmbr of unobs sets that occurred in p.season (over which to predict)
    obs.sets.ALL <- obs.sets.OUT + obs.sets.IN
    # unobserved sets broken out by category for the year
    unobs.sets.OUT <- sum(unobs.sets[yr,OUT.ctgs])         
    unobs.sets.IN <- sum(unobs.sets[yr,IN.ctgs])       
    unobs.sets.ALL <- unobs.sets.OUT + unobs.sets.IN
    # observed + unobserved sets broken out by category for the year
    tot.sets.OUT <- obs.sets.OUT + unobs.sets.OUT
    tot.sets.IN <- obs.sets.IN + unobs.sets.IN
    tot.sets.ALL <- tot.sets.OUT + tot.sets.IN
    # proportions of sets in different categories
    pr.obs.IN <- obs.sets.IN/obs.sets.ALL               # proportion of all observed sets that are INSIDE specs
    pr.obs.OUT <- obs.sets.OUT/obs.sets.ALL             # proportion of all observed sets that are OUTSIDE specs
    pr.unobs.IN <- unobs.sets.IN/unobs.sets.ALL         # proportion of all unobserved sets that are INSIDE specs
    pr.unobs.OUT <- unobs.sets.OUT/unobs.sets.ALL       # proportion of all unobserved sets that are OUTSIDE specs
    pr.tot.IN <- (obs.sets.IN + unobs.sets.IN)/tot.sets.ALL  # proportion of ALL sets that are INSIDE specs
    pr.tot.OUT <- (obs.sets.OUT + unobs.sets.OUT)/tot.sets.ALL  # proprotion of ALL sets that are OUTSIDE specs
    # calculate recent 5-year average effort in each category  
    Ntot.OUT.avg <- round(sum(tot.sets[16:20,OUT.ctgs])/5)  # average *total* effort (nmbr of sets) *outside* intxn from 2005 through 2009 (conservative estimate of avg total effort in future seasons)
    Ntot.IN.avg <-  round(sum(tot.sets[16:20,IN.ctgs])/5) # average *total* effort (nmbr of sets) *inside* intxn from 2005 through 2009 (conservative estimate of avg total effort in future seasons) 
  }
#** Models M5 and M5z  
  if (mdl=="aM5"|mdl=="aM5z") {
    obs.sets1 <- effort.summ.df$Obs.C1[yr]       # nmbr of obs sets that occurred in p.season (over which to predict)
    obs.sets2 <- effort.summ.df$Obs.C2[yr]       # nmbr of obs sets that occurred in p.season (over which to predict)
    obs.sets3 <- effort.summ.df$Obs.C3[yr]
    obs.sets4 <- effort.summ.df$Obs.C4[yr]
    obs.sets.ALL <- obs.sets1 + obs.sets2 + obs.sets3 + obs.sets4
    unobs.sets1 <- effort.summ.df$Unobs.C1[yr]    # nmbr of unobs sets that occurred in p.season (over which to predict)
    unobs.sets2 <- effort.summ.df$Unobs.C2[yr]    # nmbr of unobs sets that occurred in p.season (over which to predict)
    unobs.sets3 <- effort.summ.df$Unobs.C3[yr]
    unobs.sets4 <- effort.summ.df$Unobs.C4[yr]
    unobs.sets.ALL <- unobs.sets1 + unobs.sets2  + unobs.sets3  + unobs.sets4
    tot.sets1 <- effort.summ.df$Tot.C1[yr]        # TOTAL nmbr of sets (obs + unobs) that occurred in p.season (over which to predict)
    tot.sets2 <- effort.summ.df$Tot.C2[yr]        # TOTAL nmbr of sets (obs + unobs) that occurred in p.season (over which to predict)
    tot.sets3 <- effort.summ.df$Tot.C3[yr]
    tot.sets4 <- effort.summ.df$Tot.C4[yr]
    tot.sets.ALL <- tot.sets1 + tot.sets2 + tot.sets3 + tot.sets4
    # proportions of sets in different categories
    pr.obs1 <- obs.sets1/obs.sets.ALL             # proportion of all observed sets that are in Catgy 1 specs
    pr.obs2 <- obs.sets2/obs.sets.ALL             # proportion of all observed sets that are in Catgy 2 specs
    pr.obs3 <- obs.sets3/obs.sets.ALL             # proportion of all observed sets that are in Catgy 3 specs
    pr.obs4 <- obs.sets4/obs.sets.ALL             # proportion of all observed sets that are in Catgy 4 specs
    pr.unobs1 <- unobs.sets1/unobs.sets.ALL       # proportion of all unobserved sets that are in Catgy 1 specs
    pr.unobs2 <- unobs.sets2/unobs.sets.ALL       # proportion of all unobserved sets that are in Catgy 2 specs
    pr.unobs3 <- unobs.sets3/unobs.sets.ALL       # proportion of all unobserved sets that are in Catgy 3 specs
    pr.unobs4 <- unobs.sets4/unobs.sets.ALL       # proportion of all unobserved sets that are in Catgy 4 specs
    pr.tot1 <- tot.sets1/tot.sets.ALL             # proportion of total sets that are in Catgy 1 specs
    pr.tot2 <- tot.sets2/tot.sets.ALL             # proportion of total sets that are in Catgy 2 specs
    pr.tot3 <- tot.sets3/tot.sets.ALL             # proportion of total sets that are in Catgy 3 specs
    pr.tot4 <- tot.sets4/tot.sets.ALL             # proportion of total sets that are in Catgy 4 specs
    # calculate recent 5-year average effort in each category  
    Ntot1.avg <- round(sum(effort.summ.df$Tot.C1[16:20])/5)      # average *total* effort (nmbr of sets) from 2005 through 2009 (conservative estimate of avg total effort in future seasons)
    Ntot2.avg <- round(sum(effort.summ.df$Tot.C2[16:20])/5)      # average *total* effort (nmbr of sets) from 2005 through 2009 (conservative estimate of avg total effort in future seasons)
    Ntot3.avg <- round(sum(effort.summ.df$Tot.C3[16:20])/5)      # average *total* effort (nmbr of sets) from 2005 through 2009 (conservative estimate of avg total effort in future seasons)
    Ntot4.avg <- round(sum(effort.summ.df$Tot.C4[16:20])/5)      # average *total* effort (nmbr of sets) from 2005 through 2009 (conservative estimate of avg total effort in future seasons)
  }
#** ALL models  
  # number of takes and kills for the year
  if (sp == "DC") { 
    n.obs.takes <- takes.DC.tot[yr]                   # for DC, total number of takes for the prediction season 
    takes.vec <- takes.DC.tot                         # for DC, vector of observed takes for all seasons
    n.obs.kills <- kills.DC.tot[yr]                   # total number of mortalities/serious injuries
  }
  if (sp == "MN") {
    n.obs.takes <- takes.MN.tot[yr]                   # for MN, total number of takes for the prediction season               
    takes.vec <- takes.MN.tot                         # for MN, vector of observed takes for all seasons
    n.obs.kills <- kills.MN.tot[yr]                   # total number of mortalities/serious injuries
  }
  
# START PREDICTION SIMULATION
# Simulation to get a distribution of the predicted nmbr of total takes (y) and mortalities/injuries (z) for the season
  for (i in 1:s) {
    # Randomly sample from MCMC posteriors for parameters estimated by the model
    r1 <- sample(x=1:len.tra, size=1,replace=FALSE)          # a number between 1 and 10,000 randomly drawn
    # For each specific model, parameters and equation for theta will differ
    # Randomly sample from MCMC posteriors for parameters estimated by the model
    # Note 11/18/13: realized that randomly pulling from each parameter's estimated distribution creates too large a range on theta that is calculated from those draws
    # so instead, now just randomly pulling a row from the MCMC output and taking the value for each parameter from that row since those numbers were produced together  
#** START: Model M1 - all data categories in 1 group ###
    if (mdl=="aM1") {
      theta.s[[yr]][i] <- tracer[[1]][,2][r1]       # theta.s = simulated theta posterior from MCMC (only uses 1st MCMC chain)
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over
      lam.s.obs[[yr]][i] <- theta.s[[yr]][i] * obs.sets.ALL       # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- theta.s[[yr]][i] * unobs.sets.ALL       # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i] <- theta.s[[yr]][i] * tot.sets.ALL          # predicted total expected annual take rate per season, given actual nmbr of sets for each season (updated this: used to be Ntot.avg: avg sets 2005-2009)  
    }   
#** START: Models M1z and M1r - data split into pre- and post-regulation (1990-2000 and 2001-2009) ###
    if (mdl=="aM1z") {
      p1.s[[yr]][i] <- tracer[[1]][,2][r1]          # p1 for pre-closure (only uses 1st MCMC chain)
      p2.s[[yr]][i] <- tracer[[1]][,3][r1]          # p2 for post-closure (only uses 1st MCMC chain)
      theta.s[[yr]][i] <- tracer[[1]][,4][r1]       # theta.s = simulated theta posterior from MCMC (only uses 1st MCMC chain)
      if (p.season < 2001)  {p.s <- p1.s[[yr]][i]}
      if (p.season >= 2001) {p.s <- p2.s[[yr]][i]}
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over
      lam.s.obs[[yr]][i] <- p.s * theta.s[[yr]][i] * obs.sets.ALL           # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- p.s * theta.s[[yr]][i] * unobs.sets.ALL       # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i] <- p.s * theta.s[[yr]][i] * tot.sets.ALL           # predicted total expected annual take rate per season, given actual nmbr of sets for each season (updated this: used to be Ntot.avg: avg sets 2005-2009)
    }
    if (mdl=="aM1r") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]      # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b1.s[[yr]][i] <- tracer[[1]][,2][r1]      # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      if (p.season < 2001)  {C.s <- 0}          # closure dummy: 0 if closure not in effect, 1 if closure in effect
      if (p.season >= 2001) {C.s <- 1}
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*C.s)
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over
      lam.s.obs[[yr]][i] <- theta.s[[yr]][i] * obs.sets.ALL         # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- theta.s[[yr]][i] * unobs.sets.ALL     # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i] <- theta.s[[yr]][i] * tot.sets.ALL         # predicted total expected annual take rate per season, given actual nmbr of sets for each season (updated this: used to be Ntot.avg: avg sets 2005-2009)      
    }
#** START: Models M2, M2z, M3, M3z, M4, M4z - data split into 2 categories (based on combos of C1, C2, C3, C4) ###
    if (mdl=="aM2") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]      # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b1.s[[yr]][i] <- tracer[[1]][,2][r1]      # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0)   # outside the area; including p1.s/p2.s (ZIP param) here, it will be carried thru all lambda calcs
      theta2.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1)   # inside the area         
    }
    if (mdl=="aM2z") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]      # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b1.s[[yr]][i] <- tracer[[1]][,2][r1]      # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      p1.s[[yr]][i] <- tracer[[1]][,4][r1]      # p1 for pre-closure (only uses 1st MCMC chain)
      p2.s[[yr]][i] <- tracer[[1]][,5][r1]      # p2 for post-closure (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- p1.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0)   # outside the area; including p1.s/p2.s (ZIP param) here, it will be carried thru all lambda calcs
      theta2.s[[yr]][i] <- p2.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1)   # inside the area   
    }
    if (mdl=="aM3") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]        # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b2.s[[yr]][i] <- tracer[[1]][,2][r1]        # simulated b2 posterior from MCMC (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- exp(b0.s[[yr]][i] + b2.s[[yr]][i]*0)   # not during the time closure
      theta2.s[[yr]][i] <- exp(b0.s[[yr]][i] + b2.s[[yr]][i]*1)   # during the time closure   
    }
    if (mdl=="aM3z") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]        # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b2.s[[yr]][i] <- tracer[[1]][,2][r1]        # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      p1.s[[yr]][i] <- tracer[[1]][,4][r1]        # p1 for pre-closure (only uses 1st MCMC chain)
      p2.s[[yr]][i] <- tracer[[1]][,5][r1]        # p2 for post-closure (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- p1.s[[yr]][i] * exp(b0.s[[yr]][i] + b2.s[[yr]][i]*0)   # not during the time closure
      theta2.s[[yr]][i] <- p2.s[[yr]][i] * exp(b0.s[[yr]][i] + b2.s[[yr]][i]*1)   # during the time closure    
    }
    if (mdl=="aM4") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]      # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b3.s[[yr]][i] <- tracer[[1]][,2][r1]      # simulated b3 posterior from MCMC (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- exp(b0.s[[yr]][i] + b3.s[[yr]][i]*0)        # not during the time closure
      theta2.s[[yr]][i] <- exp(b0.s[[yr]][i] + b3.s[[yr]][i]*1)        # during the time closure   
    }
    if (mdl=="aM4z") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]  # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b3.s[[yr]][i] <- tracer[[1]][,2][r1]  # simulated b3 posterior from MCMC (only uses 1st MCMC chain)
      p1.s[[yr]][i] <- tracer[[1]][,4][r1]  # p1 for outside time/area interaction combo (only uses 1st MCMC chain)
      p2.s[[yr]][i] <- tracer[[1]][,5][r1]  # p2 for inside time/area interaction combo(only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- p1.s[[yr]][i] * exp(b0.s[[yr]][i] + b3.s[[yr]][i]*0)        # not during the time/area interaction closure
      theta2.s[[yr]][i] <- p2.s[[yr]][i] * exp(b0.s[[yr]][i] + b3.s[[yr]][i]*1)        # during the time/area interaction closure   
    }
    if (mdl=="aM2"|mdl=="aM2z"|mdl=="aM3"|mdl=="aM3z"|mdl=="aM4"|mdl=="aM4z") {
      theta.s[[yr]][i] <- theta1.s[[yr]][i]*pr.tot.OUT + theta2.s[[yr]][i]*pr.tot.IN  # thetas are multiplied by proportion of total sets in that category
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over and proportion of sets in that category
      lam.s.obs[[yr]][i]   <- theta1.s[[yr]][i]*obs.sets.OUT*pr.obs.OUT     + theta2.s[[yr]][i]*obs.sets.IN*pr.obs.IN           # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- theta1.s[[yr]][i]*unobs.sets.OUT*pr.unobs.OUT + theta2.s[[yr]][i]*unobs.sets.IN*pr.unobs.IN     # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i]   <- theta1.s[[yr]][i]*tot.sets.OUT*pr.tot.OUT     + theta2.s[[yr]][i]*tot.sets.IN*pr.tot.IN    # note 11/18/13: this calculation seems more straightforward
    }
#** START: Models M5 and M5z ###    
    if (mdl=="aM5") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]    # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b1.s[[yr]][i] <- tracer[[1]][,2][r1]    # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      b2.s[[yr]][i] <- tracer[[1]][,3][r1]    # simulated b2 posterior from MCMC (only uses 1st MCMC chain)
      b3.s[[yr]][i] <- tracer[[1]][,4][r1]    # simulated b3 posterior from MCMC (only uses 1st MCMC chain)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0 + b2.s[[yr]][i]*0 + b3.s[[yr]][i]*0)   # area= no, time = no
      theta2.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1 + b2.s[[yr]][i]*0 + b3.s[[yr]][i]*0)   # area = yes, time = no
      theta3.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0 + b2.s[[yr]][i]*1 + b3.s[[yr]][i]*0)   # area = no, time = yes
      theta4.s[[yr]][i] <- exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1 + b2.s[[yr]][i]*1 + b3.s[[yr]][i]*1)   # area = yes, time = yes
      theta.s[[yr]][i] <- theta1.s[[yr]][i]*pr.tot1 + theta2.s[[yr]][i]*pr.tot2 + theta3.s[[yr]][i]*pr.tot3 + theta4.s[[yr]][i]*pr.tot4 # thetas are multiplied by proportion of total sets in that category
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over
      lam.s.obs[[yr]][i] <-   theta1.s[[yr]][i]*obs.sets1*pr.obs1 + theta2.s[[yr]][i]*obs.sets2*pr.obs2 + 
                              theta3.s[[yr]][i]*obs.sets3*pr.obs3 + theta4.s[[yr]][i]*obs.sets4*pr.obs4        # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- theta1.s[[yr]][i]*unobs.sets1*pr.unobs1 + theta2.s[[yr]][i]*unobs.sets2*pr.unobs2 +
                              theta3.s[[yr]][i]*unobs.sets3*pr.unobs3 + theta4.s[[yr]][i]*unobs.sets4*pr.unobs4    # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i] <-   theta1.s[[yr]][i]*tot.sets1*pr.tot1 + theta2.s[[yr]][i]*tot.sets2*pr.tot2 +
                              theta3.s[[yr]][i]*tot.sets3*pr.tot3 + theta4.s[[yr]][i]*tot.sets4*pr.tot4          # predicted total expected annual take rate per season, given actual nmbr of sets for each season (updated this: used to be Ntot.avg: avg sets 2005-2009)      
    }
    if (mdl=="aM5z") {
      b0.s[[yr]][i] <- tracer[[1]][,1][r1]    # simulated b0 posterior from MCMC (only uses 1st MCMC chain)
      b1.s[[yr]][i] <- tracer[[1]][,2][r1]    # simulated b1 posterior from MCMC (only uses 1st MCMC chain)
      b2.s[[yr]][i] <- tracer[[1]][,3][r1]    # simulated b2 posterior from MCMC (only uses 1st MCMC chain)
      b3.s[[yr]][i] <- tracer[[1]][,4][r1]    # simulated b3 posterior from MCMC (only uses 1st MCMC chain)
      p1.s[[yr]][i] <- tracer[[1]][,6][r1]    # simulated p posterior (ZIP)
      p2.s[[yr]][i] <- tracer[[1]][,7][r1]    # simulated p posterior (ZIP)
      p3.s[[yr]][i] <- tracer[[1]][,8][r1]    # simulated p posterior (ZIP)
      p4.s[[yr]][i] <- tracer[[1]][,9][r1]    # simulated p posterior (ZIP)
      # Plug parameters into equation for theta (if parameter isn't theta itself)
      theta1.s[[yr]][i] <- p1.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0 + b2.s[[yr]][i]*0 + b3.s[[yr]][i]*0)   # area= no, time = no
      theta2.s[[yr]][i] <- p2.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1 + b2.s[[yr]][i]*0 + b3.s[[yr]][i]*0)   # area = yes, time = no
      theta3.s[[yr]][i] <- p3.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*0 + b2.s[[yr]][i]*1 + b3.s[[yr]][i]*0)   # area = no, time = yes
      theta4.s[[yr]][i] <- p4.s[[yr]][i] * exp(b0.s[[yr]][i] + b1.s[[yr]][i]*1 + b2.s[[yr]][i]*1 + b3.s[[yr]][i]*1)   # area = yes, time = yes
      theta.s[[yr]][i] <- theta1.s[[yr]][i]*pr.tot1 + theta2.s[[yr]][i]*pr.tot2 + theta3.s[[yr]][i]*pr.tot3 + theta4.s[[yr]][i]*pr.tot4 # thetas are multiplied by proportion of total sets in that category
      # Plug theta into the equation for lambda (Poisson parameter) by multiplying it by the number of sets to predict over
      lam.s.obs[[yr]][i] <-   theta1.s[[yr]][i]*obs.sets1*pr.obs1 + theta2.s[[yr]][i]*obs.sets2*pr.obs2 + 
                              theta3.s[[yr]][i]*obs.sets3*pr.obs3 + theta4.s[[yr]][i]*obs.sets4*pr.obs4        # take rate theta * nmbr of sets in that category that will be observed in the prediction year
      lam.s.unobs[[yr]][i] <- theta1.s[[yr]][i]*unobs.sets1*pr.unobs1 + theta2.s[[yr]][i]*unobs.sets2*pr.unobs2 + 
                              theta3.s[[yr]][i]*unobs.sets3*pr.unobs3 + theta4.s[[yr]][i]*unobs.sets4*pr.unobs4    # take rate theta * nmbr of sets in that category that will be unobserved in the prediction year
      lam.s.tot[[yr]][i] <-   theta1.s[[yr]][i]*tot.sets1*pr.tot1 + theta2.s[[yr]][i]*tot.sets2*pr.tot2 +
                              theta3.s[[yr]][i]*tot.sets3*pr.tot3 + theta4.s[[yr]][i]*tot.sets4*pr.tot4          # predicted total expected annual take rate per season, given actual nmbr of sets for each season (updated this: used to be Ntot.avg: avg sets 2005-2009)      
    }
### BELOW THIS POINT SHOULD BE THE SAME FOR ALL THE MODELS ###
  # Plug lambda into Poisson model (number of takes predicted for the number of sets for the season is Poisson distributed and characterized by the parameter lambda, which is take rate theta times exposure number of sets)
    y.s.obs[[yr]][i] <- rpois(n=1, lambda=lam.s.obs[[yr]][i])           # take n=1 random draw from poisson dist; y = predicted/simulated number of observed takes    
    y.s.unobs[[yr]][i] <- rpois(n=1, lambda=lam.s.unobs[[yr]][i])       # take n=1 random draw from poisson dist; y = predicted/simulated number of unobserved takes    
  # Draw a mortality rate 'q' from beta-binomial prior distribution
    if (sp == "DC") q.sim[[yr]][i] <- rbeta(n=1, shape1=15, shape2=11)  # n = nmbr of random draws from beta distribution; shape parameters were determined by Steve using DC data(see BetaExpl spreadsheet)
    if (sp == "MN") q.sim[[yr]][i] <- rbeta(n=1, shape1=2, shape2=4)    # n = nmbr of random draws from beta distribution; shape parameters were determined by Steve using MN data(see BetaExpl spreadsheet)
  # Multiply mortality rate 'q' by lambda to get the simulated draw from the posterior predictive distribution of the rate of mortality/serious injury('mr.ppd')
    mr.s.obs[[yr]][i] <- q.sim[[yr]][i] * lam.s.obs[[yr]][i]            # mr = posterior predictive distribution of the rate of mortality/serious injury per season for the number of sets being predicted over 
    mr.s.unobs[[yr]][i] <- q.sim[[yr]][i] * lam.s.unobs[[yr]][i]        # mr = posterior predictive distribution of the rate of mortality/serious injury per season for the number of sets being predicted over 
    mr.s.tot[[yr]][i] <- q.sim[[yr]][i] * lam.s.tot[[yr]][i]            # predicted total expected annual mortality                  
    # Plug 'q' and 'y' into Binomial distribution to get number of mortalities 'z' given number of takes 'y'
    z.s.obs[[yr]][i] <- rbinom(n=1, size= round(y.s.obs[[yr]][i]), prob= q.sim[[yr]][i])        # z = number of observed mortalities for 'y' number of takes in prediction year
    z.s.unobs[[yr]][i] <- rbinom(n=1, size= round(y.s.unobs[[yr]][i]), prob= q.sim[[yr]][i])    # z = number of unobserved mortalities for 'y' number of takes in prediction year
  
  # Create Vectors for Hypothetical future season *expected annual mortality* prediction (using avg total sets 2005-2009) -- CALCULATION FOR ONE YEAR ONLY
    if (yr == 20) {                                                     # use 2009 posterior predictive distribution
      if (mdl=="aM1"|mdl=="aM1r"|mdl=="aM1z") {
        if (mdl=="aM1"|mdl=="aM1r") {lam.s.HypoFutSsn[i] <- theta.s[[yr]][i] * Ntot.avg}       # predicted total expected annual take rate per season, given Ntot.avg: avg sets 2005-2009)
        if (mdl=="aM1z")            {lam.s.HypoFutSsn[i] <- p.s * theta.s[[yr]][i] * Ntot.avg}
        mr.s.HypoFutSsn[i] <- q.sim[[yr]][i] * lam.s.HypoFutSsn[i]    # predicted total expected annual mortality              
      }
      if (mdl=="aM2"|mdl=="aM2z"|mdl=="aM3"|mdl=="aM3z"|mdl=="aM4"|mdl=="aM4z") {
          lam.s.HypoFutSsn[i] <- theta1.s[[yr]][i]*pr.tot.OUT*Ntot.OUT.avg + theta2.s[[yr]][i]*pr.tot.IN*Ntot.IN.avg       # predicted total expected annual take rate per season, given Ntot.avg for each category: avg sets 2005-2009)
          mr.s.HypoFutSsn[i] <- q.sim[[yr]][i] * lam.s.HypoFutSsn[i]    # predicted total expected annual mortality             
      }
      if (mdl=="aM5"|mdl=="aM5z") {
        lam.s.HypoFutSsn[i] <- theta1.s[[yr]][i]*Ntot1.avg + theta2.s[[yr]][i]*Ntot2.avg + theta3.s[[yr]][i]*Ntot3.avg + theta4.s[[yr]][i]*Ntot4.avg    # predicted total expected annual take rate per season, given Ntot.avg for each category: avg sets 2005-2009)
        mr.s.HypoFutSsn[i] <- q.sim[[yr]][i] * lam.s.HypoFutSsn[i]    # predicted total expected annual mortality              
      }
    }                       
  } # **** END PREDICTION SIMULATION for one season **** ---------------------------------------------------------------  
  
  # CALCULATE CUMULATIVE PROBABILITY CURVE TO GET 99% and 50% LIMITS on TAKES, MORTALITY RATE, and MORTALITIES
  th.s.99 <- cumul.prob.fxn(theta.s[[yr]], s=s)$UpperLim99CI        # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on *theta* = take rate per set
  th.s.50 <- cumul.prob.fxn(theta.s[[yr]], s=s)$Median50            # use cumul.prob.fxn to calculate the 50% median/central tendency on *theta* = take rate per set
  
  lam.s.99.obs <- cumul.prob.fxn(lam.s.obs[[yr]], s=s)$UpperLim99CI      # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on observed *lambda* = take rate per season
  lam.s.50.obs <- cumul.prob.fxn(lam.s.obs[[yr]], s=s)$Median50          # use cumul.prob.fxn to calculate the 50% median/central tendency on observed *lambda* = take rate per season
  lam.s.99.unobs <- cumul.prob.fxn(lam.s.unobs[[yr]], s=s)$UpperLim99CI      # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on unobserved *lambda* = take rate per season
  lam.s.50.unobs <- cumul.prob.fxn(lam.s.unobs[[yr]], s=s)$Median50          # use cumul.prob.fxn to calculate the 50% median/central tendency on unobserved *lambda* = take rate per season
  lam.s.99.tot <- cumul.prob.fxn(lam.s.tot[[yr]], s=s)$UpperLim99CI          # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on total *lambda* = take rate per season, given avg total sets 2005-2009
  lam.s.50.tot <- cumul.prob.fxn(lam.s.tot[[yr]], s=s)$Median50              # use cumul.prob.fxn to calculate the 50% median/central tendency on total *lambda* = take rate per season, given avg total sets 2005-2009
  
  y.s.99.obs <- cumul.prob.fxn(y.s.obs[[yr]], s=s)$UpperLim99CI          # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the number of *observed takes*
  y.s.50.obs <- cumul.prob.fxn(y.s.obs[[yr]], s=s)$Median50              # use cumul.prob.fxn to calculate the 50% median/central tendency on the number of *observed takes*
  y.s.99.unobs <- cumul.prob.fxn(y.s.unobs[[yr]], s=s)$UpperLim99CI          # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the number of *unobserved takes*
  y.s.50.unobs <- cumul.prob.fxn(y.s.unobs[[yr]], s=s)$Median50              # use cumul.prob.fxn to calculate the 50% median/central tendency on the number of *unobserved takes*
  
  mr.s.99.obs <- cumul.prob.fxn(mr.s.obs[[yr]], s=s)$UpperLim99CI        # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the *observed mortality rate per season*
  mr.s.50.obs <- cumul.prob.fxn(mr.s.obs[[yr]], s=s)$Median50            # use cumul.prob.fxn to calculate the 50% median/central tendency on the *observed mortality rate per season*   
  mr.s.99.unobs <- cumul.prob.fxn(mr.s.unobs[[yr]], s=s)$UpperLim99CI        # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the *unobserved mortality rate per season*
  mr.s.50.unobs <- cumul.prob.fxn(mr.s.unobs[[yr]], s=s)$Median50            # use cumul.prob.fxn to calculate the 50% median/central tendency on the *unobserved mortality rate per season*   
  mr.s.99.tot <- cumul.prob.fxn(mr.s.tot[[yr]], s=s)$UpperLim99CI            # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the *total mortality rate per season*, given avg total sets 2005-2009       
  mr.s.50.tot <- cumul.prob.fxn(mr.s.tot[[yr]], s=s)$Median50                # use cumul.prob.fxn to calculate the 50% median/central tendency on the *total mortality rate per season*, given avg total sets 2005-2009         
  
  z.s.99.obs <- cumul.prob.fxn(z.s.obs[[yr]], s=s)$UpperLim99CI      # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the number of *observed mortalities*
  z.s.50.obs <- cumul.prob.fxn(z.s.obs[[yr]], s=s)$Median50          # use cumul.prob.fxn to calculate the 50% median/central tendency on the number of *observed mortalities*
  z.s.99.unobs <- cumul.prob.fxn(z.s.unobs[[yr]], s=s)$UpperLim99CI      # use cumul.prob.fxn to calculate the upper limit of the 99% credibility interval on the number of *unobserved mortalities*
  z.s.50.unobs <- cumul.prob.fxn(z.s.unobs[[yr]], s=s)$Median50          # use cumul.prob.fxn to calculate the 50% median/central tendency on the number of *unobserved mortalities*
  
  # STORE predictions in data frame in general order of: observed, unobserved, total for the different values 
  predictions.df[yr,] <- c(p.season, obs.sets.ALL, unobs.sets.ALL, n.obs.takes, n.obs.kills,
                           th.s.99, th.s.50, lam.s.99.obs, lam.s.50.obs,lam.s.99.unobs, lam.s.50.unobs, lam.s.99.tot, lam.s.50.tot,
                           y.s.99.obs, y.s.50.obs, y.s.99.unobs, y.s.50.unobs, y.s.99.unobs+n.obs.takes, y.s.50.unobs+n.obs.takes,
                           mr.s.99.obs, mr.s.50.obs, mr.s.99.unobs, mr.s.50.unobs, mr.s.99.tot, mr.s.50.tot,
                           z.s.99.obs, z.s.50.obs, z.s.99.unobs, z.s.50.unobs, z.s.99.unobs+n.obs.kills, z.s.50.unobs+n.obs.kills) 
} # END loop through seasons

predictions.df

# write the summary output results to a .csv file
assign(paste(mdl,"predictions",sp, sep=""), value= predictions.df, envir = .GlobalEnv)
today <- paste(substr(date(),9,10), substr(date(),5,7), substr(date(),23,24),sep="")       # extract Month-Day-Year from automatic date function -- to add into exported file name below
filename <- paste(sp, "-MCMC-",mdl,"-predictions-",today ,".csv", sep="")
write.csv(x=predictions.df, file=filename, row.names=FALSE)  

# write the lambda and mortality rate vectors to files
assign(paste("lam.s.HypoFutSsn.",mdl,".",sp,sep=""),value=lam.s.HypoFutSsn, envir=.GlobalEnv)
assign(paste("mr.s.HypoFutSsn.",mdl,".",sp,sep=""),value=mr.s.HypoFutSsn, envir=.GlobalEnv)
lam.out <- paste(sp, "-LTTR-", mdl,"-",today ,".csv", sep="")
write.csv(x=lam.s.HypoFutSsn, file=lam.out, row.names=TRUE)     
mr.out <- paste(sp, "-LTMR-", mdl,"-",today ,".csv", sep="")
write.csv(x=mr.s.HypoFutSsn, file=mr.out, row.names=TRUE)    

# ***********************************************
# ***********************************************
# End PREDICTIONS by season 1990-2009








